@@ -5,6 +5,7 @@ __call__ method.
 """
 import re
 import logging
+from collections import defaultdict
 from types import SimpleNamespace
 from typing import Sequence, List, Tuple, Optional, Set
 from abc import ABC, abstractmethod
@@ -20,6 +21,7 @@ from .adapters import (
     Match,
     remainder,
     Adapter,
+    AdapterIndex,
 )
 from .tokenizer import tokenize_braces, TokenizeError, Token, BraceToken
 from .info import ModificationInfo
@@ -161,9 +163,9 @@ class AdapterCutter(SingleEndModifier):
         suffix: List[SingleAdapter] = []
         other: List[SingleAdapter] = []
         for a in adapters:
-            if IndexedPrefixAdapters.is_acceptable(a):
+            if AdapterIndex.is_acceptable(a, prefix=True):
                 prefix.append(a)
-            elif IndexedSuffixAdapters.is_acceptable(a):
+            elif AdapterIndex.is_acceptable(a, prefix=False):
                 suffix.append(a)
             else:
                 other.append(a)
@@ -303,6 +305,103 @@ class ReverseComplementer(SingleEndModifier):
         return trimmed_read
 
 
+class PairedReverseComplementer(PairedEndModifier):
+    """Trim adapters from a read pair or its reverse complement (R1/R2 swapped)"""
+
+    def __init__(
+        self,
+        adapter_cutter1: Optional[AdapterCutter],
+        adapter_cutter2: Optional[AdapterCutter],
+        rc_suffix: Optional[str] = " rc",
+    ):
+        """
+        rc_suffix -- suffix to add to the read name if sequence was reverse-complemented
+        """
+        self.adapter_cutter1 = adapter_cutter1
+        self.adapter_cutter2 = adapter_cutter2
+        self.reverse_complemented = 0  # counter
+        self._suffix = rc_suffix
+
+    def __repr__(self):
+        return (
+            "PairedReverseComplementer("
+            f"adapter_cutter1={self.adapter_cutter1}, adapter_cutter2={self.adapter_cutter2})"
+        )
+
+    def __call__(
+        self,
+        r1: SequenceRecord,
+        r2: SequenceRecord,
+        info1: ModificationInfo,
+        info2: ModificationInfo,
+    ):
+        # Run normally
+        if self.adapter_cutter1 is not None:
+            r1_trimmed, r1_matches = self.adapter_cutter1.match_and_trim(r1)
+        else:
+            r1_trimmed, r1_matches = r1, []
+        if self.adapter_cutter2 is not None:
+            r2_trimmed, r2_matches = self.adapter_cutter2.match_and_trim(r2)
+        else:
+            r2_trimmed, r2_matches = r2, []
+        unswapped_score = sum(m.score for m in r1_matches) + sum(
+            m.score for m in r2_matches
+        )
+
+        # Run with R1 and R2 swapped (equivalent to reverse complementing)
+        if self.adapter_cutter1 is not None:
+            (
+                r1_trimmed_swapped,
+                r1_matches_swapped,
+            ) = self.adapter_cutter1.match_and_trim(r2)
+        else:
+            r1_trimmed_swapped, r1_matches_swapped = r2, []
+        if self.adapter_cutter2 is not None:
+            (
+                r2_trimmed_swapped,
+                r2_matches_swapped,
+            ) = self.adapter_cutter2.match_and_trim(r1)
+        else:
+            r2_trimmed_swapped, r2_matches_swapped = r1, []
+        swapped_score = sum(m.score for m in r1_matches_swapped) + sum(
+            m.score for m in r2_matches_swapped
+        )
+
+        # Compare and pick the variant that is better
+        use_reverse_complement = swapped_score > unswapped_score
+
+        if use_reverse_complement:
+            self.reverse_complemented += 1
+            r1_trimmed = r1_trimmed_swapped
+            r2_trimmed = r2_trimmed_swapped
+            r1_matches = r1_matches_swapped
+            r2_matches = r2_matches_swapped
+            info1.is_rc = info2.is_rc = True
+            if self._suffix:
+                r1_trimmed.name += self._suffix
+                r2_trimmed.name += self._suffix
+        else:
+            info1.is_rc = info2.is_rc = False
+
+        if r1_matches:
+            self.adapter_cutter1.with_adapters += 1  # type: ignore
+            for match in r1_matches:
+                stats = self.adapter_cutter1.adapter_statistics[match.adapter]  # type: ignore
+                stats.add_match(match)
+                stats.reverse_complemented += bool(use_reverse_complement)
+            info1.matches.extend(r1_matches)  # TODO extend or overwrite?
+
+        if r2_matches:
+            self.adapter_cutter2.with_adapters += 1  # type: ignore
+            for match in r2_matches:
+                stats = self.adapter_cutter2.adapter_statistics[match.adapter]  # type: ignore
+                stats.add_match(match)
+                stats.reverse_complemented += bool(use_reverse_complement)
+            info2.matches.extend(r2_matches)  # TODO extend or overwrite?
+
+        return r1_trimmed, r2_trimmed
+
+
 class PairedAdapterCutterError(Exception):
     pass
 
@@ -322,7 +421,6 @@ class PairedAdapterCutter(PairedEndModifier):
 
         action -- What to do with a found adapter: None, 'trim', 'lowercase' or 'mask'
         """
-        super().__init__()
         if len(adapters1) != len(adapters2):
             raise PairedAdapterCutterError(
                 "The number of adapters to trim from R1 and R2 must be the same. "
@@ -532,6 +630,9 @@ class Renamer(SingleEndModifier):
     def __repr__(self):
         return f"{self.__class__.__name__}('{self._template}')"
 
+    def __reduce__(self):
+        return Renamer, (self._template,)
+
     def compile_rename_function(self):
         """
         Create the function that computes a new name
@@ -626,7 +727,6 @@ class PairedEndRenamer(PairedEndModifier):
         info1: ModificationInfo,
         info2: ModificationInfo,
     ) -> Tuple[SequenceRecord, SequenceRecord]:
-
         if not record_names_match(read1.name, read2.name):
             id1 = Renamer.parse_name(read1.name)[0]
             id2 = Renamer.parse_name(read1.name)[1]
@@ -653,7 +753,6 @@ class PairedEndRenamer(PairedEndModifier):
         info1: ModificationInfo,
         info2: ModificationInfo,
     ) -> Tuple[str, str]:
-
         id1, comment1 = Renamer.parse_name(read1.name)
         id2, comment2 = Renamer.parse_name(read2.name)
         header1 = read1.name
@@ -753,16 +852,24 @@ class QualityTrimmer(SingleEndModifier):
 
 
 class PolyATrimmer(SingleEndModifier):
-    def __init__(self):
-        self.trimmed_bases = 0
+    """Trim poly-A tails or poly-T heads"""
+
+    def __init__(self, revcomp=False):
+        self.trimmed_bases = defaultdict(int)
+        self.revcomp = revcomp
 
     def __repr__(self):
         return "PolyATrimmer()"
 
     def __call__(self, record: SequenceRecord, info: ModificationInfo):
-        index = poly_a_trim_index(record.sequence)
-        self.trimmed_bases += len(record) - index
-        return record[:index]
+        if self.revcomp:
+            index = poly_a_trim_index(record.sequence, revcomp=True)
+            self.trimmed_bases[index] += 1
+            return record[index:]
+        else:
+            index = poly_a_trim_index(record.sequence)
+            self.trimmed_bases[len(record) - index] += 1
+            return record[:index]
 
 
 class Shortener(SingleEndModifier):
