@@ -28,11 +28,11 @@ def test_help():
     assert e.value.args[0] == 0
 
 
-def test_unknown_file_format(tmp_path):
+def test_unknown_file_format(tmp_path, cores):
     path = tmp_path / "unknown_format.txt"
     path.write_text("raw text")
     with pytest.raises(SystemExit):
-        main([str(path)])
+        main([f"--cores={cores}", str(path)])
 
 
 def test_cores_negative():
@@ -267,19 +267,19 @@ def test_twoadapters(run):
 def test_poly_a_legacy(run):
     """poly-A tails"""
     run(
-        "-m 24 -O 10 -a AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
-        "polya.fasta",
-        "polya.fasta",
+        "-O 10 -a AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA",
+        "polya.legacy.1.fasta",
+        "polya.1.fasta",
     )
 
 
 def test_poly_a_legacy_brace_notation(run):
     """poly-A tails"""
-    run("-m 24 -O 10 -a A{35}", "polya.fasta", "polya.fasta")
+    run("-O 10 -a A{35}", "polya.legacy.1.fasta", "polya.1.fasta")
 
 
 def test_poly_a(run):
-    run("--poly-a", "poly-a.fasta", "polya.fasta")
+    run("--poly-a", "polya.1.fasta", "polya.1.fasta")
 
 
 # the same as --action=none
@@ -1084,7 +1084,7 @@ def test_duplicate_output_paths(tmp_path):
     # "specified more than once as an output file"
 
 
-def test_rename(run):
+def test_rename(run, cores):
     run(
         [
             "--rename={id}_{cut_suffix} {header} {adapter_name}",
@@ -1093,6 +1093,8 @@ def test_rename(run):
             "OnlyT=TTTTTT",
             "-a",
             "OnlyG=GGGGGG",
+            "--cores",
+            str(cores),
         ],
         "rename.fastq",
         "suffix.fastq",
