@@ -714,7 +714,7 @@ def test_combinatorial_demultiplexing(tmp_path, discarduntrimmed, cores):
     else:
         combinations.extend((a, b, True) for a, b in optional)
     main(params)
-    for (name1, name2, should_exist) in combinations:
+    for name1, name2, should_exist in combinations:
         for i in (1, 2):
             name = "combinatorial.{name1}_{name2}.{i}.fastq".format(
                 name1=name1, name2=name2, i=i
@@ -762,3 +762,65 @@ def test_rename(run_paired, cores):
         expected2="rename.2.fastq",
         cores=cores,
     )
+
+
+def test_poly_a_poly_t(run_paired, cores):
+    run_paired(
+        "--poly-a",
+        in1="polya.1.fasta",
+        in2="polya.2.fasta",
+        expected1="polya.1.fasta",
+        expected2="polya.2.fasta",
+        cores=cores,
+    )
+
+
+def test_revcomp_only_r1(run_paired):
+    run_paired(
+        [
+            "--revcomp",
+            "-g",
+            "^TTATTTGTCT",
+            "-g",
+            "^TCCGCACTGGC",
+        ],
+        in1="revcomp.1.fastq",
+        in2="revcomp.2.fastq",
+        expected1="revcomp.1.fastq",
+        expected2="revcomp.2.fastq",
+        cores=1,
+    )
+
+
+def test_revcomp_only_r2(run_paired):
+    run_paired(
+        [
+            "--revcomp",
+            "-G",
+            "^TTATTTGTCT",
+            "-G",
+            "^TCCGCACTGGC",
+        ],
+        in1="revcomp.2.fastq",
+        in2="revcomp.1.fastq",
+        expected1="revcomp.2.fastq",
+        expected2="revcomp.1.fastq",
+        cores=1,
+    )
+
+
+def test_revcomp_r1_and_r2(run_paired):
+    run_paired(
+        [
+            "--revcomp",
+            "-g",
+            "^TTATTTGTCT",
+            "-G",
+            "^TCCGCACTGGC",
+        ],
+        in1="revcomp.1.fastq",
+        in2="revcomp.2.fastq",
+        expected1="revcomp-r1r2.1.fastq",
+        expected2="revcomp-r1r2.2.fastq",
+        cores=1,
+    )
